package com.sams.jxta.groups.student;

import net.jxta.credential.Credential;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peer.PeerID;
import net.jxta.id.ID;
import net.jxta.membership.MembershipService;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.Element;

/*
 * This class represents a Credential that is supplied to a
 * peer after a successful apply process.
 */
public class InvitationLetter implements Credential {
    // Reference to the Membership Service
    private UniversityAdmissionsService admissionsService;

   /*
    * Constructor which takes a Membership Service Object
    */
    public InvitationLetter(UniversityAdmissionsService admissionsService){
                            
        this.admissionsService = admissionsService;
    }
    /*
     * Getter for the Membership Service
     */
    public MembershipService  getSourceService(){
        
        return admissionsService;
    }

    /*
     * This method returns the PeerGroup ID
     */
    public ID getPeerGroupID(){
        return admissionsService.getPeerGroup().getPeerGroupID();
    }
    
    /*
     * This method returns the Peer ID
     */
    public ID getPeerID(){
        return admissionsService.getPeerGroup().getPeerID();
    }

    /*
     * This method returns a Structured Document representing the Credential
     */
    public StructuredDocument getDocument(MimeMediaType as) throws Exception {
        
            StructuredDocument doc =
                StructuredDocumentFactory.newStructuredDocument( as,"InivtationLetter" );
            
            Element e = doc.createElement( "PeerGroupID", 
                    admissionsService.getPeerGroup().getPeerGroupID() );
            doc.appendChild( e );
            e = doc.createElement("TimeOfRequest",""+System.currentTimeMillis());
            doc.appendChild( e );
            return doc;
    }
}

